<?php

namespace Drupal\Tests\ui_patterns_field_formatters\Kernel;

use Drupal\Component\Plugin\PluginBase;
use Drupal\Component\Render\FormattableMarkup;
use Drupal\Core\Render\Markup;

/**
 * Render test.
 *
 * @group ui_patterns_field_formatters
 */
class RenderTest extends UIPatternsFieldFormattersTestBase {
  /**
   * {@inheritdoc}
   */
  protected $strictConfigSchema = FALSE;

  /**
   * Test render.
   *
   * @param string $field_name
   *   Field name.
   * @param array $field_items
   *   Field items.
   * @param bool $is_slot
   *   Is a slot or prop.
   * @param string $type
   *   Type.
   * @param string $component_id
   *   Component Id.
   * @param string $key_map
   *   Key map.
   * @param array $source_settings
   *   Plugin.
   * @param string $field_property
   *   Source id.
   * @param string $expected_output
   *   Expected output.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   *
   * @dataProvider providerFieldMapping
   */
  public function testRender(
    string $field_name,
    array $field_items,
    bool $is_slot,
    string $type,
    string $component_id,
    string $key_map,
    array $source_settings,
    string $field_property,
    string $expected_output,
  ) : void {
    foreach ($this->entityTypes as $oneEntityType) {
      $bundle = $oneEntityType->id();
      $entity_type_id = $oneEntityType->getEntityType()->getBundleOf();
      // Create node page with body.
      $source_id = ($field_property == "") ?
          implode(PluginBase::DERIVATIVE_SEPARATOR, ["field_formatter", $entity_type_id, $bundle, $field_name]) :
          implode(PluginBase::DERIVATIVE_SEPARATOR, ["field_property", $entity_type_id, $field_name, $field_property]);
      $props_data = [];
      $slots_data = [];
      if ($is_slot) {
        $slots_data[$key_map] = [
          [
            "source" => $source_settings,
            "source_id" => $source_id,
          ],
        ];
      }
      else {
        $props_data[$key_map] = [
          "source" => $source_settings,
          "source_id" => $source_id,
        ];
      }

      $display_component_settings = [
        'label' => 'hidden',
        'type' => $type,
        'region' => 'content',
        'settings' => [
          'ui_patterns' => $this->getComponentSettings($component_id, $props_data, $slots_data),
        ],
      ];
      $this->setUpFieldDisplay($entity_type_id, (string) $bundle, $field_name, $display_component_settings);
      $node = $this->drupalCreateNode(
        array_merge([
          'type' => $bundle,
        ], $field_items)
      );
      foreach ($field_items as $field_items_field_name => $field_items_per_field_name) {
        $node->set($field_items_field_name, $field_items_per_field_name);
      }
      $produced = $this->renderEntity($node);
      // $produced = $rendered[$field_name]['#markup'];
      $produced = $this->normalizeMarkupString($produced);
      $expected_output = $this->normalizeMarkupString($expected_output);
      $error_string = new FormattableMarkup("failed to verify produced markup for @formatter.\nOutput is:\n---\n@output\n---\nExpected is:\n---\n@expected\n---\nFormatter settings is:\n @settings.", [
        '@formatter' => $type,
        '@output' => Markup::create($produced),
        '@expected' => Markup::create($expected_output),
        '@settings' => Markup::create(print_r($display_component_settings, TRUE)),
      ]);
      $this->assertEquals($expected_output, $produced, $error_string);
    }
  }

  /**
   * Callback data provider to test one slot, one prop.
   *
   * @return array[]
   *   Data provider for function testRender.
   */
  public function providerFieldMapping(): array {
    /*
    field_formatter:node::nid
    field_formatter:node::uuid
    field_formatter:node::vid
    field_formatter:node::langcode
    field_formatter:node::type
    field_formatter:node::revision_timestamp
    field_formatter:node::revision_uid
    field_formatter:node::revision_log
    field_formatter:node::status
    field_formatter:node::uid
    field_formatter:node::title
    field_formatter:node::created
    field_formatter:node::changed
    field_formatter:node::promote
    field_formatter:node::sticky
    field_formatter:node::default_langcode
    field_formatter:node::revision_default
    field_formatter:node::revision_translation_affected
    field_formatter:node::field_text_with_summary_1
    field_formatter:node::field_text_with_summary
    field_formatter:node::field_text_1
    field_formatter:node::field_text
    field_formatter:node::field_text_long_1
    field_formatter:node::field_text_long
    field_formatter:node::field_datetime_1
    field_formatter:node::field_datetime
    field_formatter:node::field_integer_1
    field_formatter:node::field_integer
    field_formatter:node::field_uri_1
    field_formatter:node::field_uri
    field_formatter:node::field_entity_reference_1
    field_formatter:node::field_entity_reference
    field_formatter:node::field_uuid_1
    field_formatter:node::field_uuid
    field_formatter:node::field_float_1
    field_formatter:node::field_float
    field_formatter:node::field_email_1
    field_formatter:node::field_email
    field_formatter:node::field_timestamp_1
    field_formatter:node::field_timestamp
    field_formatter:node::field_string_1
    field_formatter:node::field_string
    field_formatter:node::field_language_1
    field_formatter:node::field_language
    field_formatter:node::field_created_1
    field_formatter:node::field_created
    field_formatter:node::field_changed_1
    field_formatter:node::field_changed
    field_formatter:node::field_map_1
    field_formatter:node::field_map
    field_formatter:node::field_string_long_1
    field_formatter:node::field_string_long
    field_formatter:node::field_password_1
    field_formatter:node::field_password
    field_formatter:node::field_decimal_1
    field_formatter:node::field_decimal
    field_formatter:node::field_boolean_1
    field_formatter:node::field_boolean
     */
    $tests_per_prop_type = [];

    // Add some tests for different field types
    // and their natively associated prop types.
    $data_per_type = [
      ['field_type' => 'boolean', 'field_items' => [TRUE], 'prop_type' => 'boolean', 'field_property' => 'value'],
      ['field_type' => 'float', 'field_items' => [1.1, 2.0, -12], 'prop_type' => 'number', 'field_property' => 'value'],
      // This will require an automatic conversion.
      ['field_type' => 'float', 'field_items' => [1.1, 2.0, -12], 'prop_type' => 'string', 'field_property' => 'value'],
      ['field_type' => 'decimal', 'field_items' => [1, 100, -10], 'prop_type' => 'number', 'field_property' => 'value'],
      // This will require an automatic conversion.
      ['field_type' => 'decimal', 'field_items' => [1, 100, -10], 'prop_type' => 'string', 'field_property' => 'value'],
      ['field_type' => 'text', 'field_items' => ["hello"], 'prop_type' => 'string', 'field_property' => 'value'],
    ];
    foreach ($data_per_type as $one_data) {
      $prop_type = $one_data['prop_type'];
      foreach ($one_data['field_items'] as $field_item) {
        $casted_field_value = "" . $field_item;
        $field_value = '<div data-component-id="ui_patterns_tests_components:multi_prop_type_test" class="prop-' . $prop_type . '">' . $casted_field_value . '</div>';
        $field_name = sprintf('field_%s', $one_data['field_type']);
        $tests_per_prop_type[] = [
          'field_name' => $field_name,
          'field_items' => [$field_name => [$field_item]],
          'is_slot' => FALSE,
          'type' => 'ui_patterns_component_per_item',
          'component_id' => 'ui_patterns_tests_components:multi_prop_type_test',
          'key_map' => sprintf('prop_%s', $prop_type),
          'source_settings' => [],
          'field_property' => $one_data['field_property'],
          'expected' => '<div><div>' . $field_value . '</div></div>',
        ];
      }
    }
    // Add tests with a body field and field formatters.
    $tests_formatters = [
      // ui_patterns_component_per_item / multiple items.
      [
        'field_name' => 'field_text_with_summary',
        'field_items' => ['field_text_with_summary' => ['this is first body', 'this is second body']],
        'is_slot' => TRUE,
        'type' => 'ui_patterns_component_per_item',
        'component_id' => 'ui_patterns_tests_components:slot_test',
        'key_map' => 'my_slot',
        'source_settings' => ["type" => "text_default"],
        'field_property' => '',
        'expected' => <<<EXPECTED
    <div>
    <div>
      <div data-component-id="ui_patterns_tests_components:slot_test" class="test-class">
        <p>this is first body</p>
      </div>
    </div>
    <div>
      <div data-component-id="ui_patterns_tests_components:slot_test" class="test-class">
        <p>this is second body</p>
      </div>
    </div>
    </div>
    EXPECTED,
      ],
      // Test 2.
      [
        'field_name' => 'field_text_with_summary',
        'field_items' => ['field_text_with_summary' => ['this is first body2', 'this is second body2']],
        'is_slot' => TRUE,
        'type' => 'ui_patterns_component',
        'component_id' => 'ui_patterns_tests_components:slot_test',
        'key_map' => 'my_slot',
        'source_settings' => ["type" => "text_default"],
        'field_property' => '',
        'expected' => <<<EXPECTED
<div>
              <div><div data-component-id="ui_patterns_tests_components:slot_test" class="test-class"><p>this is first body2</p>
<p>this is second body2</p>
</div></div>
          </div>
EXPECTED,
      ],
      // Test 4.
      [
        'field_name' => 'field_text_with_summary',
        'field_items' => ['field_text_with_summary' => ['this is first body3', 'this is second body3']],
        'is_slot' => FALSE,
        'type' => 'ui_patterns_component_per_item',
        'component_id' => 'ui_patterns_tests_components:string_prop_type_test',
        'key_map' => 'string_prop',
        'source_settings' => [],
        'field_property' => 'value',
        'expected' => <<<EXPECTED
<div>
              <div><div data-component-id="ui_patterns_tests_components:string_prop_type_test" class="prop-test">this is first body3</div></div>
              <div><div data-component-id="ui_patterns_tests_components:string_prop_type_test" class="prop-test">this is second body3</div></div>
          </div>
EXPECTED,
      ],
      // Edge case, this should not be possible in theory.
      [
        'field_name' => 'field_text_with_summary',
        'field_items' => ['field_text_with_summary' => ['this is first body4', 'this is second body4']],
        'is_slot' => FALSE,
        'type' => 'ui_patterns_component',
        'component_id' => 'ui_patterns_tests_components:string_prop_type_test',
        'key_map' => 'string_prop',
        'source_settings' => [],
        'field_property' => 'value',
        'expected' => <<<EXPECTED
<div>
              <div><div data-component-id="ui_patterns_tests_components:string_prop_type_test" class="prop-test"></div></div>
          </div>
EXPECTED,
      ],
    ];

    $tests_cardinality = [
      [
        'field_name' => 'field_text_1',
        'field_items' => ['field_text_1' => ['single value']],
        'is_slot' => TRUE,
        'type' => 'ui_patterns_component_per_item',
        'component_id' => 'ui_patterns_tests_components:slot_test',
        'key_map' => 'my_slot',
        'source_settings' => ["type" => "text_default"],
        'field_property' => '',
        'expected' => <<<EXPECTED
      <div>
        <div data-component-id="ui_patterns_tests_components:slot_test" class="test-class">
          <p>single value</p>
        </div>
      </div>
      EXPECTED,
      ],
      // This should not work, so default formatter is used...
      [
        'field_name' => 'field_text_1',
        'field_items' => ['field_text_1' => ['single value']],
        'is_slot' => TRUE,
        'type' => 'ui_patterns_component',
        'component_id' => 'ui_patterns_tests_components:slot_test',
        'key_map' => 'my_slot',
        'source_settings' => ["type" => "text_default"],
        'field_property' => '',
        'expected' => <<<EXPECTED
      <div><p>single value</p></div>
      EXPECTED,
      ],
    ];

    $tests_nesting = [
      // Field granularity propagation to ui_patterns_component formatter.
      [
        'field_name' => 'field_text',
        'field_items' => ['field_text' => ['1', '2', '3']],
        'is_slot' => TRUE,
        'type' => 'ui_patterns_component_per_item',
        'component_id' => 'ui_patterns_tests_components:slot_test',
        'key_map' => 'my_slot',
        'source_settings' => [
          "type" => "ui_patterns_component",
          "settings" => [
            "ui_patterns" => [
              "component_id" => "ui_patterns_tests_components:slot_test",
              "variant_id" => NULL,
              "props" => [],
              "slots" => [
                "my_slot" => [
                  "sources" => [
                    [
                      "source" => [
                        "type" => "text_default",
                      ],
                      "source_id" => "field_property:node:field_text:value",
                    ],
                  ],
                ],
              ],
            ],
          ],
        ],
        'field_property' => '',
        'expected' => <<<EXPECTED
    <div>
      <div>
        <div data-component-id="ui_patterns_tests_components:slot_test" class="test-class">
           <div data-component-id="ui_patterns_tests_components:slot_test" class="test-class">
           1
           </div>
        </div>
      </div>
      <div>
        <div data-component-id="ui_patterns_tests_components:slot_test" class="test-class">
           <div data-component-id="ui_patterns_tests_components:slot_test" class="test-class">
           2
           </div>
        </div>
      </div>
      <div>
        <div data-component-id="ui_patterns_tests_components:slot_test" class="test-class">
           <div data-component-id="ui_patterns_tests_components:slot_test" class="test-class">
           3
           </div>
        </div>
      </div>
    </div>
    EXPECTED,
      ],
      [
        'field_name' => 'field_text',
        'field_items' => ['field_text' => ['1', '2', '3']],
        'is_slot' => TRUE,
        'type' => 'ui_patterns_component_per_item',
        'component_id' => 'ui_patterns_tests_components:slot_test',
        'key_map' => 'my_slot',
        'source_settings' => [
          "type" => "ui_patterns_component_per_item",
          "settings" => [
            "ui_patterns" => [
              "component_id" => "ui_patterns_tests_components:slot_test",
              "variant_id" => NULL,
              "props" => [],
              "slots" => [
                "my_slot" => [
                  "sources" => [
                    [
                      "source" => [
                        "type" => "text_default",
                      ],
                      "source_id" => "field_property:node:field_text:value",
                    ],
                  ],
                ],
              ],
            ],
          ],
        ],
        'field_property' => '',
        'expected' => <<<EXPECTED
    <div>
      <div>
        <div data-component-id="ui_patterns_tests_components:slot_test" class="test-class">
           <div data-component-id="ui_patterns_tests_components:slot_test" class="test-class">
           1
           </div>
        </div>
      </div>
      <div>
        <div data-component-id="ui_patterns_tests_components:slot_test" class="test-class">
           <div data-component-id="ui_patterns_tests_components:slot_test" class="test-class">
           2
           </div>
        </div>
      </div>
      <div>
        <div data-component-id="ui_patterns_tests_components:slot_test" class="test-class">
           <div data-component-id="ui_patterns_tests_components:slot_test" class="test-class">
           3
           </div>
        </div>
      </div>
    </div>
    EXPECTED,
      ],
      [
        'field_name' => 'field_text',
        'field_items' => ['field_text' => ['1', '2', '3']],
        'is_slot' => TRUE,
        'type' => 'ui_patterns_component',
        'component_id' => 'ui_patterns_tests_components:slot_test',
        'key_map' => 'my_slot',
        'source_settings' => [
          "type" => "ui_patterns_component_per_item",
          "settings" => [
            "ui_patterns" => [
              "component_id" => "ui_patterns_tests_components:slot_test",
              "variant_id" => NULL,
              "props" => [],
              "slots" => [
                "my_slot" => [
                  "sources" => [
                    [
                      "source" => [
                        "type" => "text_default",
                      ],
                      "source_id" => "field_property:node:field_text:value",
                    ],
                  ],
                ],
              ],
            ],
          ],
        ],
        'field_property' => '',
        'expected' => <<<EXPECTED
    <div>
      <div>
        <div data-component-id="ui_patterns_tests_components:slot_test" class="test-class">
           <div data-component-id="ui_patterns_tests_components:slot_test" class="test-class">
           1
           </div>
           <div data-component-id="ui_patterns_tests_components:slot_test" class="test-class">
           2
           </div>
           <div data-component-id="ui_patterns_tests_components:slot_test" class="test-class">
           3
           </div>
        </div>
      </div>
    </div>
    EXPECTED,
      ],
      // field_property can't work here.
      [
        'field_name' => 'field_text',
        'field_items' => ['field_text' => ['1', '2', '3']],
        'is_slot' => TRUE,
        'type' => 'ui_patterns_component',
        'component_id' => 'ui_patterns_tests_components:slot_test',
        'key_map' => 'my_slot',
        'source_settings' => [
          "type" => "ui_patterns_component",
          "settings" => [
            "ui_patterns" => [
              "component_id" => "ui_patterns_tests_components:slot_test",
              "variant_id" => NULL,
              "props" => [],
              "slots" => [
                "my_slot" => [
                  "sources" => [
                    [
                      "source" => [
                        "type" => "text_default",
                      ],
                      "source_id" => "field_property:node:field_text:value",
                    ],
                  ],
                ],
              ],
            ],
          ],
        ],
        'field_property' => '',
        'expected' => <<<EXPECTED
    <div>
      <div>
        <div data-component-id="ui_patterns_tests_components:slot_test" class="test-class">
           <div data-component-id="ui_patterns_tests_components:slot_test" class="test-class">
           
           </div>
        </div>
      </div>
    </div>
    EXPECTED,
      ],
    ];

    return array_merge($tests_nesting, $tests_cardinality, $tests_per_prop_type, $tests_formatters);
  }

}
