<?php

namespace Drupal\Tests\ui_patterns_layouts\Kernel;

use Drupal\Component\Render\FormattableMarkup;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Entity\Display\EntityDisplayInterface;
use Drupal\Core\Entity\Entity\EntityFormDisplay;
use Drupal\Core\Entity\Entity\EntityViewDisplay;
use Drupal\Core\Render\Markup;
use Drupal\field_layout\Display\EntityDisplayWithLayoutInterface;
use Drupal\Tests\ui_patterns\Kernel\EntityTestBase;
use Symfony\Component\Yaml\Yaml;

/**
 * Defines a base class for node access kernel tests.
 */
abstract class UIPatternsLayoutsTestBase extends EntityTestBase {

  /**
   * Additional modules to activate (child classes should set this).
   *
   * @var array
   */
  protected static array $additionalModules = [];
  /**
   * {@inheritdoc}
   */
  protected $strictConfigSchema = FALSE;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [];

  /**
   * Config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Test render.
   *
   * @param string $field_name
   *   Field name.
   * @param array $field_items
   *   Field items.
   * @param array|null $display_component_settings
   *   Display component settings.
   * @param array $third_party_settings
   *   Third party settings.
   * @param string $expected_output
   *   Expected output.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function doTestRender(
    string $field_name,
    array $field_items,
    ?array $display_component_settings,
    array $third_party_settings,
    string $expected_output,
  ) : void {
    foreach ($this->entityTypes as $oneEntityType) {
      $bundle = (string) $oneEntityType->id();
      $entity_type_id = $oneEntityType->getEntityType()->getBundleOf();
      $this->doTestRenderBundle($entity_type_id, $bundle, $field_name, $field_items, $display_component_settings, $third_party_settings, $expected_output);
    }
  }

  /**
   * Test render.
   *
   * @param string $entity_type_id
   *   Entity type id.
   * @param string $bundle
   *   Bundle.
   * @param string $field_name
   *   Field name.
   * @param array $field_items
   *   Field items.
   * @param array|null $display_component_settings
   *   Display component settings.
   * @param array $third_party_settings
   *   Third party settings.
   * @param string $expected_output
   *   Expected output.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function doTestRenderBundle(
    string $entity_type_id,
    string $bundle,
    string $field_name,
    array $field_items,
    ?array $display_component_settings,
    array $third_party_settings,
    string $expected_output,
  ) : void {
    $this->setUpFieldDisplay($entity_type_id, (string) $bundle, $field_name, $display_component_settings, $third_party_settings);
    $node = $this->drupalCreateNode(
        array_merge([
          'type' => $bundle,
        ], $field_items)
      );
    foreach ($field_items as $field_items_field_name => $field_items_per_field_name) {
      $node->set($field_items_field_name, $field_items_per_field_name);
    }
    $produced = $this->renderEntity($node);
    $produced = $this->normalizeMarkupString($produced);
    $expected_output = $this->normalizeMarkupString($expected_output);

    $config = $this->configFactory->get(sprintf("core.entity_view_display.%s.%s.default", $entity_type_id, $bundle));
    $config = Yaml::dump($config->get(), 10, 2, Yaml::DUMP_MULTI_LINE_LITERAL_BLOCK);
    $error_string = new FormattableMarkup("failed to verify produced markup for entity render.\nOutput is:\n---\n@output\n---\nExpected is:\n---\n@expected\n---\nFormatter settings is:\n @settings.\n @third_party_settings.\n @config.", [
      '@output' => Markup::create($produced),
      '@expected' => Markup::create($expected_output),
      '@settings' => Markup::create(print_r($display_component_settings, TRUE)),
      '@third_party_settings' => Markup::create(print_r($third_party_settings, TRUE)),
      '@config' => Markup::create(print_r($config, TRUE)),
    ]);
    $this->assertEquals($expected_output, $produced, $error_string);
  }

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    self::$modules = array_merge(parent::$modules, ['field_layout', 'layout_discovery', 'ui_patterns_layouts'], static::$additionalModules);
    parent::setUp();
    $this->installConfig('field_layout');
    $entity_save = function (EntityDisplayInterface $entity) {
      if ($entity instanceof EntityDisplayWithLayoutInterface) {
        $entity->ensureLayout()->save();
      }
    };
    array_map($entity_save, EntityViewDisplay::loadMultiple());
    array_map($entity_save, EntityFormDisplay::loadMultiple());

    // Invalidate the render cache since all content will now have a layout.
    Cache::invalidateTags(['rendered']);
    $this->configFactory = $this->container->get('config.factory');
  }

}
