<?php

namespace Drupal\Tests\ui_patterns\Kernel;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityDisplayRepositoryInterface;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\field\Entity\FieldConfig;
use Drupal\field\Entity\FieldStorageConfig;
use Drupal\field\FieldConfigInterface;
use Drupal\KernelTests\KernelTestBase;
use Drupal\node\Entity\NodeType;
use Drupal\Tests\node\Traits\NodeCreationTrait;
use Drupal\Tests\user\Traits\UserCreationTrait;
use Drupal\user\Entity\Role;
use Drupal\user\Entity\User;
use org\bovigo\vfs\vfsStreamWrapper;

/**
 * Defines a base class for node access kernel tests.
 */
abstract class EntityTestBase extends KernelTestBase {

  use NodeCreationTrait {
    createNode as drupalCreateNode;
  }
  use UserCreationTrait {
    createUser as drupalCreateUser;
  }

  /**
   * {@inheritdoc}
   */
  protected string $defaultTheme = 'ui_patterns_test_theme';


  /**
   * Default node bundle.
   *
   * @var string
   */
  protected static string $defaultNodeBundle = 'page';

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'user',
    'system',
    'sdc',
    'field',
    'text',
    'datetime',
    'filter',
    'ui_patterns',
    'ui_patterns_test',
    'ui_patterns_tests_components',
    'node',
  ];

  /**
   * The source plugin manager.
   *
   * @var \Drupal\ui_patterns\SourcePluginManager
   */
  protected $sourceManager;

  /**
   * The field type plugin manager.
   *
   * @var \Drupal\Core\Field\FieldTypePluginManager
   */
  protected $fieldTypeManager;

  /**
   * Renderer.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * Display repository.
   *
   * @var \Drupal\Core\Entity\EntityDisplayRepositoryInterface
   */
  protected $displayRepository;

  /**
   * Entity Type Manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;


  /**
   * Entity Type.
   *
   * @var array<\Drupal\node\NodeTypeInterface>
   */
  protected $entityTypes = [];

  /**
   * Fields.
   *
   * @var array<\Drupal\field\FieldConfigInterface>
   */
  protected $fields;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    vfsStreamWrapper::register();
    parent::setUp();
    $this->installSchema('user', 'users_data');
    $this->installConfig(['system', 'user']);
    $this->installEntitySchema('user');
    // $this->drupalCreateUser();
    $this->installConfig('filter');
    $this->installEntitySchema('date_format');
    $this->fieldTypeManager = $this->container->get('plugin.manager.field.field_type');
    $this->sourceManager = $this->container->get('plugin.manager.ui_patterns_source');
    $this->setUpNode();
    $this->displayRepository = $this->container->get('entity_display.repository');
    $this->renderer = $this->container->get('renderer');
    $this->entityTypeManager = $this->container->get('entity_type.manager');
    \Drupal::service('theme_installer')
      ->install([
        'ui_patterns_test_theme',
      ]);
    $this->config('system.theme')->set('default', 'ui_patterns_test_theme')->save();

    // Create a role with the desired permission.
    $role = Role::create([
      'id' => 'custom_role',
      'label' => 'Custom role',
    ]);
    // Example permission.
    $role->grantPermission('access content');
    $role->save();

    // Create a user and assign the role.
    $user = User::create([
      'name' => 'test_user',
      'status' => 1,
    ]);
    $user->addRole((string) $role->id());
    $user->save();

    // Set the current user to the created user.
    \Drupal::service('account_switcher')->switchTo($user);
    // \Drupal::currentUser()->setAccount($user);
  }

  /**
   * Set up the node with fields.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  private function setUpNode() : void {
    $this->installEntitySchema('node');
    $this->installConfig(['field', 'node']);
    // $this->randomMachineName();
    $bundle = self::$defaultNodeBundle;
    $type = NodeType::load($bundle);
    if (!$type) {
      $type = NodeType::create([
        'name' => $this->randomString(10),
        'type' => $bundle,
      ]);
      $type->save();
    }
    $this->entityTypes[] = $type;

    $field_types = $this->fieldTypeManager->getDefinitions();

    foreach (array_keys($field_types) as $field_type_id) {
      $field_name = sprintf("field_%s_%s", $field_type_id, 1);
      $bundle = (string) $type->id();
      $this->fields[$field_name] =
        $this->createEntityField($type->getEntityType()->getBundleOf(), $bundle, $field_name, $field_type_id, 1);
      $field_name = sprintf("field_%s", $field_type_id);
      $this->fields[$field_name] =
        $this->createEntityField($type->getEntityType()->getBundleOf(), $bundle, $field_name, $field_type_id,
          FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED);
    }
  }

  /**
   * Create a field on a content type.
   */
  protected function createEntityField(
    string $entity_type,
    string $bundle,
    string $field_name,
    string $field_type_id,
    int $cardinality = FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED,
  ) : FieldConfigInterface {
    $field_type_definition = $this->fieldTypeManager->getDefinition($field_type_id);
    // Create a field storage.
    $field_storage = [
      'field_name' => $field_name,
      'entity_type' => $entity_type,
      'type' => $field_type_id,
      'settings' => [],
      'cardinality' => $cardinality,
    ];
    FieldStorageConfig::create($field_storage)->save();
    // Create a field instance on the content type.
    $field = [
      'field_name' => $field_storage['field_name'],
      'entity_type' => $entity_type,
      'bundle' => $bundle,
      'label' => $field_type_definition['label'],
      'settings' => [],
    ];
    $field_config = FieldConfig::create($field);
    $field_config->save();
    // Set cardinality.
    $field_storage_reload = FieldStorageConfig::loadByName($entity_type, $field_name);
    $field_storage_reload->setCardinality($cardinality);
    $field_storage_reload->save();
    return $field_config;
  }

  /**
   * Normalize a string of markup for comparison.
   */
  protected function normalizeMarkupString(string $markup): string {
    $markup = preg_replace('/\s+/', ' ', $markup);
    $markup = preg_replace('/\s*>\s*/', '>', $markup);
    $markup = preg_replace('/\s*</', '<', $markup);
    $markup = str_replace("\n", "", $markup);
    $markup = trim($markup);
    return $markup;
  }

  /**
   * Create a component settings.
   *
   * @param string $component_id
   *   The component id.
   * @param array $props_data
   *   Array with source and source_id.
   * @param array $slots_data
   *   Array of arrays with source and source_id.
   *
   * @return array<string, array|string>
   *   The component Settings
   */
  protected function getComponentSettings(string $component_id, array $props_data, array $slots_data) : array {
    $ui_patterns = [
      'component_id' => $component_id,
      'variant_id' => '',
      'slots' => [],
      'props' => [],
    ];
    foreach ($slots_data as $key_map => $slot_data_sources_for_slot) {
      $ui_patterns["slots"][$key_map] = [
        "sources" => $slot_data_sources_for_slot,
        /*
         "sources" => [
            [
              "source" => $source_settings,
              "source_id" => $source_id,
            ],
          ],
         */
      ];
    }
    foreach ($props_data as $key_map => $prop_data_sources_for_prop) {
      $ui_patterns["props"][$key_map] = $prop_data_sources_for_prop;
    }
    return $ui_patterns;
  }

  /**
   * Render an entity.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The entity.
   * @param string $view_mode
   *   (optional) The view mode. Defaults to self::DEFAULT_DISPLAY_MODE.
   */
  public function renderEntity(
    ContentEntityInterface $entity,
    string $view_mode = EntityDisplayRepositoryInterface::DEFAULT_DISPLAY_MODE,
  ) : mixed {
    /** @var \Drupal\Core\Entity\EntityViewBuilderInterface $viewBuilder */
    $viewBuilder = $this->entityTypeManager->getViewBuilder($entity->getEntityTypeId());
    $build = $viewBuilder->view($entity, $view_mode);
    // $entity_display = $this->displayRepository->getViewDisplay(
    // $entity->getEntityTypeId(), $entity->bundle(), $view_mode);
    // $build = $entity_display->build($entity);
    $this->renderer->renderRoot($build);
    return (string) $build['#markup'];
  }

  /**
   * Set up the node with a field display.
   *
   * @param string $entity_type_id
   *   Entity type id.
   * @param string $bundle
   *   Bundle.
   * @param string $field_name
   *   Field name.
   * @param array|null $display_component_settings
   *   Display Component settings for field.
   * @param array $third_party_settings
   *   Display Component third party settings for field.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  protected function setUpFieldDisplay(string $entity_type_id, string $bundle, string $field_name, ?array $display_component_settings, array $third_party_settings = []) : void {
    $entity_display = $this->displayRepository->getViewDisplay($entity_type_id, $bundle);
    $display_components = $entity_display->getComponents();
    foreach (array_keys($display_components) as $name) {
      $entity_display->removeComponent($name);
    }
    if (is_array($display_component_settings)) {
      $entity_display->setComponent($field_name, $display_component_settings);
      foreach ($third_party_settings as $module => $third_party_settings_module) {
        foreach ($third_party_settings_module as $key => $value) {
          $entity_display->setThirdPartySetting($module, $key, $value);
        }
      }
    }
    $entity_display->save();
  }

}
